#include <gnome.h>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

// main.cpp
void set_notebook_page(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, gpointer *data);
void add_new_file(gchar *filename, gboolean fill);
void show_selected_item(GtkWidget *root_tree, GtkWidget *selected_item, gpointer *data);
void unshow_selected_item(GtkWidget *root_tree, GtkWidget *selected_item, gpointer *data);
void build_props(GtkWidget *clist, GtkWidget *text, gchar *filename);
void copy_chars(char *source, char *dest, int max_chars);
void flush_file(gchar *name);
void directory_stuff();
void save_prefs();
void adjust_sensitivity();
void adjust_props();
void set_window_title();
void display_image(gchar *image_name);
void destroy(GtkWidget *widget, gpointer data);
void kill_widget(GtkWidget *widget, GtkWidget *killer);

char gCat_dir[256] = "";
char tempfile[256] = "";
char *item_ptrs[2];

GtkWidget *window;
GtkWidget *whole_window_box;
GtkWidget *app_bar;
GtkWidget *mdi_book;

struct opened_file
{
   GtkWidget *notebook_page;
   GtkWidget *notebook_box;
   GtkWidget *notebook_label;
   GtkWidget *table;
   GtkWidget *tree_window;
   GtkWidget *tree_widget;
   GtkWidget *image_box;
   GtkWidget *pixmapwid;
   GtkWidget *main_box;
   GtkWidget *item_text;
   GtkWidget *item_scroller;
   GtkWidget *props_box;
   GtkWidget *item_list_window;
   GtkWidget *item_list;
   GtkWidget *cur_selected_item;
   GtkWidget *cur_selected_tree;
   gchar filename[256];
   gchar tempfile[256];
   gchar image_str[256];
   gboolean saved;
};

#define MAX_FILES 10
struct opened_file *file_list[MAX_FILES];
struct opened_file *cur_file;

gint total_files = 0;
gint total_props = 0;
gint cur_page = 0;

GtkWidget *file_selector;
GdkColor black;
GdkColor red;
GdkColor blue;
GdkColor green;
  
gchar *noimage_filename = "/usr/local/share/pixmaps/noimage.png";
gchar *aboutimage_filename = "/usr/local/share/pixmaps/gCatalog-about.png";


// files.cpp
void new_file(struct GtkWidget *, gpointer *data);
void file_ok_sel(GtkWidget *widget, GtkFileSelection *fs);
void open_file(GtkWidget *widget, gpointer *data);
void close_file(GtkWidget *widget, gpointer *data);
void save_file(GtkWidget *widget, gpointer *data);
void save_file_as(GtkWidget *widget, gpointer *data);
void save_as(GtkWidget *widget, GtkFileSelection *fs);
void copy_file(gchar *source, gchar *dest);


// edit.cpp
void change_text(GtkWidget *widget, gpointer *data);
void save_text(GtkWidget *widget, GtkWidget *text);
void change_image(GtkWidget *widget, gpointer *data);
void save_image(GtkWidget *widget, gpointer *data);
void insert_new_cat(GtkWidget *widget, gpointer *data);
void insert_cat(GtkWidget *widget, gpointer *data);
void insert_new_subcat(GtkWidget *widget, gpointer *data);
void insert_subcat(GtkWidget *widget, gpointer *data);
void insert_new_item(GtkWidget *widget, gpointer *data);
void insert_item(GtkWidget *widget, gpointer *data);
void item_to_category(GtkWidget *widget, gpointer *data);
void category_to_item(GtkWidget *widget, gpointer *data);
void delete_item(GtkWidget *widget, gpointer *data);

GtkWidget *change_dialog_window;
GtkWidget *main_dialog_box;
GtkWidget *edited_dialog_box;
GtkWidget *edit_buttons_dialog_box;
GtkWidget *text_dialog_box;
GtkWidget *edited_text;
GtkWidget *edited_scroller;
GtkWidget *edited_list_window;
GtkWidget *edited_list;
GtkWidget *entry_dialog_box;
GtkWidget *dialog_label;
GtkWidget *dialog_entry;
GtkWidget *dialog_label2;
GtkWidget *dialog_entry2;
GtkWidget *pixmap_entry;
GtkWidget *buttons_box;
GtkWidget *ok_button;
GtkWidget *cancel_button;


// properties.cpp
void edit_properties(GtkWidget *widget, gpointer *data);
void set_cur_sel(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data);
void set_no_sel(GtkWidget *widget, gint row, gint column, GdkEventButton *event, gpointer data);
void move_property_up(GtkWidget *widget, gpointer *data);
void move_property_down(GtkWidget *widget, gpointer *data);
void new_property(GtkWidget *widget, gpointer *data);
void do_new_prop(GtkWidget *widget, gpointer *data);
void edit_property(GtkWidget *widget, gpointer *data);
void do_edit_prop(GtkWidget *widget, gpointer *data);
void del_property(GtkWidget *widget, gpointer *data);
void save_properties(GtkWidget *widget, gpointer *data);

GtkWidget *moveup_button;
GtkWidget *movedown_button;
GtkWidget *new_button;   
GtkWidget *edit_button;
GtkWidget *del_button;

gint cur_sel_list_row;
gint cur_sel_list_col;


// options.cpp
void change_settings(int number);
void save_settings(GtkWidget *widget, gpointer *data);
void get_defaults(GtkWidget *widget, gpointer *data);
void get_backups(GtkWidget *widget, gpointer *data);
GtkWidget *make_button(gchar *button_text, gint *value, GtkWidget *which_box);
void toggle_value(GtkWidget *w, gint *value);
void write_file_to_gtktext(gchar *filename, GtkWidget *text, GdkColor *color);

GtkWidget *allow_delete;
GtkWidget *allow_edit;
GtkWidget *allow_images;
GtkWidget *allow_props;
GtkWidget *save_button;
GtkWidget *defaults_button;
GtkWidget *undo_button;

#define PREFS 4
#define ALLOW_DELETE 0
#define ALLOW_EDIT 1
#define SHOW_IMAGES 2
#define SHOW_PROPS 3
char *prefs[PREFS] = { "ALLOW_DELETE", "ALLOW_EDIT", "SHOW_IMAGES", "SHOW_PROPS" };
int default_prefs[PREFS] = { 1, 1, 1, 1 };
int backup_prefs[PREFS] = { 0, 0, 0, 0 };
int cur_prefs[PREFS] = { 0, 0, 0 , 0 };


// maketree.cpp
void make_tree(GtkWidget *tree, const char *filename);


// getprefs.cpp
void get_prefs();
void save_prefs(gboolean which);


// menus.cpp
void make_menus();
void do_prefs(GtkWidget *widget, gpointer *data);
void do_contents(GtkWidget *widget, gpointer *data);
void do_license(GtkWidget *widget, gpointer *data);
void do_about(GtkWidget *widget, gpointer *data);

GtkWidget *filemenu;
GtkWidget *editmenu;
GtkWidget *optionsmenu;
GtkWidget *helpmenu;
GtkWidget *file_menu_new;
GtkWidget *file_menu_open;
GtkWidget *file_menu_close;
GtkWidget *file_menu_save;
GtkWidget *file_menu_saveas;
GtkWidget *edit_menu_change_desc;
GtkWidget *edit_menu_change_img;
GtkWidget *edit_menu_edit_props;
GtkWidget *edit_menu_newcat;
GtkWidget *edit_menu_newsubcat;
GtkWidget *edit_menu_newitem;
GtkWidget *edit_menu_cat2item;
GtkWidget *edit_menu_item2cat;
GtkWidget *edit_menu_delete;


// toolbar.cpp
void make_toolbar();
GtkWidget *toolbar_new_button;
GtkWidget *toolbar_open_button;
GtkWidget *toolbar_close_button;
GtkWidget *toolbar_save_button;
GtkWidget *toolbar_saveas_button;
GtkWidget *toolbar_properties_button;

#include "maketree.cpp"   
#include "files.cpp"
#include "properties.cpp"
#include "edit.cpp"
#include "getprefs.cpp"
#include "menus.cpp"
#include "options.cpp"
#include "toolbar.cpp"
